////////////////////////////////////////////////////////////////////////////////
// CONFIDENTIAL and PROPRIETARY software of Magewell Electronics Co., Ltd.
// Copyright (c) 2011-2019 Magewell Electronics Co., Ltd. (Nanjing)
// All rights reserved.
// This copyright notice MUST be reproduced on all authorized copies.
////////////////////////////////////////////////////////////////////////////////

#import "ViewController.h"
#import <mach/mach.h>
#import <mach/mach_time.h>
#include <pthread.h>

static uint64_t getTimestamp() {
    uint64_t time = mach_absolute_time();
    static mach_timebase_info_data_t sTimebaseInfo;
    
    if (sTimebaseInfo.denom == 0) {
        mach_timebase_info(&sTimebaseInfo);
    }
    
    long double ret = 1.L * time / 1000 * sTimebaseInfo.numer / sTimebaseInfo.denom;
    return ret / 1000;
}

static void* onVideoCaptureThreadProc(void *param) {
    ViewController *viewCtrl = (__bridge ViewController*)param;
    
    [viewCtrl onVideoCaptureProc];
    
    return NULL;
}

static void* onAudioCaptureThreadProc(void *param) {
    ViewController *viewCtrl = (__bridge ViewController*)param;
    
    [viewCtrl onAudioCaptureProc];
    
    return NULL;
}

@implementation ViewController
- (void)onVideoCaptureFrame:(BYTE *)pBuffer bufferLen:(long)bufferLen pixelBuffer:(CVPixelBufferRef) pixelBuffer  {
    //render
    if (self.viewEnable) {
        CMSampleTimingInfo timing = {kCMTimeInvalid, kCMTimeInvalid, kCMTimeInvalid};
        CMVideoFormatDescriptionRef videoInfo = NULL;
        CVReturn result = CMVideoFormatDescriptionCreateForImageBuffer(NULL, pixelBuffer, &videoInfo);

        CMSampleBufferRef sampleBuffer = NULL;
        result = CMSampleBufferCreateForImageBuffer(kCFAllocatorDefault, pixelBuffer, true, NULL, NULL, videoInfo, &timing, &sampleBuffer);
        CFRelease(videoInfo);

        CFArrayRef attachments = CMSampleBufferGetSampleAttachmentsArray(sampleBuffer, YES);
        CFMutableDictionaryRef dict = (CFMutableDictionaryRef)CFArrayGetValueAtIndex(attachments, 0);
        CFDictionarySetValue(dict, kCMSampleAttachmentKey_DisplayImmediately, kCFBooleanTrue);

        if (self.videoLayer) {
            [self.videoLayer enqueueSampleBuffer:(CMSampleBufferRef)sampleBuffer];
        }

        CFRelease(sampleBuffer);
    }

    //fps
    {
        uint64_t now = getTimestamp();
        if (self.lastTimestamp == 0) {
            self.lastTimestamp = now;
            self.videoFrameCount = 0;
        } else {
            self.videoFrameCount++;
        }

        if (now - self.lastTimestamp >= 1000) {
            self.captureFps = self.videoFrameCount * 1000.0 / (now - self.lastTimestamp);
            self.lastTimestamp = now;
            self.videoFrameCount = 0;
            
            //NSLog(@"video capture fps: %f ", self.captureFps);
        }
    }
}

- (void)onVideoCaptureProc {
    CVPixelBufferRef pixelBuffer[3] = {NULL};
    NSDictionary *pixelAttributes = @{(NSString*)kCVPixelBufferIOSurfacePropertiesKey:@{}};
    for (int i = 0; i < sizeof(pixelBuffer)/sizeof(pixelBuffer[0]); i++) {
        CVReturn result = CVPixelBufferCreate(kCFAllocatorDefault,
                                              self.width,
                                              self.height,
                                              self.pixelFormat,
                                              (__bridge CFDictionaryRef)(pixelAttributes),
                                              &pixelBuffer[i]);
    }
    int index = 0;

    // Wait Events
    MWCAP_PTR hCaptureEvent    = MWCreateEvent();
    MWCAP_PTR hTimerEvent      = MWCreateEvent();

    MW_RESULT xr;
    do {
        xr = MWStartVideoCapture(self.hChannel, hCaptureEvent);
        if (xr != MW_SUCCEEDED)
            break;

        HTIMER hTimerNotify = MWRegisterTimer(self.hChannel, hTimerEvent);
        if (hTimerNotify == 0)
            break;

        LONGLONG llBegin = 0LL;
        xr = MWGetDeviceTime(self.hChannel, &llBegin);
        if (xr != MW_SUCCEEDED)
            break;

        LONGLONG llExpireTime = llBegin;
        DWORD dwFrameDuration = self.frameDuration;

        while (self.running) {
            llExpireTime = llExpireTime + dwFrameDuration;
            xr = MWScheduleTimer(self.hChannel, hTimerNotify, llExpireTime);
            if (xr != MW_SUCCEEDED) {
                break;
            }

            DWORD dwRet = MWWaitEvent(hTimerEvent, 1000);
            if (dwRet <= 0) {
                continue;
            }

            MWCAP_VIDEO_BUFFER_INFO videoBufferInfo;
            if (MW_SUCCEEDED != MWGetVideoBufferInfo(self.hChannel, &videoBufferInfo)) {
                continue;
            }

            MWCAP_VIDEO_SIGNAL_STATUS videoSignalStatus;
            xr = MWGetVideoSignalStatus(self.hChannel, &videoSignalStatus);
            if (xr != MW_SUCCEEDED) {
                continue;
            }

            MWCAP_VIDEO_FRAME_INFO videoFrameInfo;
            xr = MWGetVideoFrameInfo(self.hChannel, videoBufferInfo.iNewestBufferedFullFrame, &videoFrameInfo);
            if (xr != MW_SUCCEEDED)
                continue;

            RECT rcSrc = {0, 0, videoSignalStatus.cx, videoSignalStatus.cy};
            
            CVPixelBufferLockBaseAddress(pixelBuffer[index],0);
            DWORD cbStride = (DWORD)CVPixelBufferGetBytesPerRowOfPlane(pixelBuffer[index], 0);
            DWORD dwFrameSize = (DWORD)CVPixelBufferGetDataSize(pixelBuffer[index]);
            BYTE* byBuffer = (BYTE *)CVPixelBufferGetBaseAddressOfPlane(pixelBuffer[index], 0);
            xr = MWCaptureVideoFrameToVirtualAddressEx(self.hChannel, MWCAP_VIDEO_FRAME_ID_NEWEST_BUFFERED, byBuffer, dwFrameSize, cbStride, FALSE, (MWCAP_PTR64)pixelBuffer,
                                                       self.fourcc, self.width, self.height,
                                                       0, 0, NULL, NULL, 0, 100, 0, 100, 0, MWCAP_VIDEO_DEINTERLACE_BLEND, MWCAP_VIDEO_ASPECT_RATIO_IGNORE, &rcSrc, NULL, 0, 0,
                                                       MWCAP_VIDEO_COLOR_FORMAT_UNKNOWN, MWCAP_VIDEO_QUANTIZATION_UNKNOWN, MWCAP_VIDEO_SATURATION_UNKNOWN);
            MWWaitEvent(hCaptureEvent, -1);
            CVPixelBufferUnlockBaseAddress(pixelBuffer[index], 0);

            [self onVideoCaptureFrame:byBuffer bufferLen:dwFrameSize pixelBuffer:pixelBuffer[index]];
            index = (index+1) % (sizeof(pixelBuffer)/sizeof(pixelBuffer[0]));
            
            MWCAP_VIDEO_CAPTURE_STATUS captureStatus;
            xr = MWGetVideoCaptureStatus(self.hChannel, &captureStatus);
        }
        
        xr = MWUnregisterTimer(self.hChannel, hTimerNotify);
        xr = MWStopVideoCapture(self.hChannel);
    } while (FALSE);

    MWCloseEvent(hCaptureEvent);
    MWCloseEvent(hTimerEvent);

    for (int i = 0; i < sizeof(pixelBuffer)/sizeof(pixelBuffer[0]); i++) {
        CFRelease(pixelBuffer[i]);
        pixelBuffer[i] = NULL;
    }
}

- (void)onAudioCaptureProc {
    MWCAP_PTR hAudioEvent   = MWCreateEvent();
    HNOTIFY hAudioNotify    = 0;

    MW_RESULT xr = MW_SUCCEEDED;
    do {
        xr = MWStartAudioCapture(self.hChannel);
        if (xr != MW_SUCCEEDED)
            break;

        hAudioNotify = MWRegisterNotify(self.hChannel, hAudioEvent, MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED);
        if (hAudioNotify == 0)
            break;
        
        while (self.running) {
            DWORD dwRet = MWWaitEvent(hAudioEvent, 1000);
            if (dwRet <= 0)
                continue;

            ULONGLONG ullStatusBits = 0LL;
            MWGetNotifyStatus(self.hChannel, hAudioNotify, &ullStatusBits);
            if (ullStatusBits & MWCAP_NOTIFY_AUDIO_FRAME_BUFFERED) {
                do {
                    MWCAP_AUDIO_CAPTURE_FRAME audioFrame;
                    xr = MWCaptureAudioFrame(self.hChannel, &audioFrame);
                    if (xr == MW_SUCCEEDED) {
                        short asAudioSamples[MWCAP_AUDIO_SAMPLES_PER_FRAME * 2];
                        for (int i = 0; i < MWCAP_AUDIO_SAMPLES_PER_FRAME; i++) {
                            short sLeft = (short)(audioFrame.adwSamples[i * 8] >> 16);
                            short sRight = (short)(audioFrame.adwSamples[i * 8 + 4] >> 16);

                            asAudioSamples[i * 2] = sLeft;
                            asAudioSamples[i * 2 + 1] = sRight;
                        }

                        MWAudioRenderPut(self.audioRender, (uint8_t*)asAudioSamples, MWCAP_AUDIO_SAMPLES_PER_FRAME * 2 * sizeof(short));
                    }
                } while (xr == MW_SUCCEEDED && self.running);
            }
        }
        
        xr = MWUnregisterNotify(self.hChannel, hAudioNotify);
        xr = MWStopAudioCapture(self.hChannel);
    } while (0);

    MWCloseEvent(hAudioEvent);
}

- (void)startCapture:(NSMenuItem *)deviceItem {
    [self stopCapture];
    
    int index = (int)[deviceItem tag];
    char wPath[256] = {0};
    MWGetDevicePath(index, wPath);
    self.hChannel = MWOpenChannelByPath(wPath);
    if (self.hChannel == NULL) {
        printf("ERROR: Open channel failed !\n");
        return;
    }
    
    self.running = true;
    
    MWAudioRenderCreate(self.samplerate, self.audioChannels, self.bitsPerSample, &_audioRender);
    MWAudioRenderStart(self.audioRender);
    
    if (self.videoCaptureThreadId == 0) {
        pthread_t tid = 0;
        if (0 == pthread_create(&tid, NULL, onVideoCaptureThreadProc,  (__bridge void*)(self))) {
            self.videoCaptureThreadId = tid;
        }
    }
    
    if (self.audioCaptureThreadId == 0) {
        pthread_t tid = 0;
        if (0 == pthread_create(&tid, NULL, onAudioCaptureThreadProc,  (__bridge void*)(self))) {
           self.audioCaptureThreadId = tid;
        }
    }
    
    [deviceItem setState:NSControlStateValueOn];
    self.selectedDeviceItem = deviceItem;
}

- (void)stopCapture {
    self.running = false;
    
    if (self.audioCaptureThreadId) {
        pthread_join(self.audioCaptureThreadId, NULL);
        self.audioCaptureThreadId = 0;
    }
    
    if (self.audioRender) {
        MWAudioRenderStop(self.audioRender);
        MWAudioRenderDestroy(self.audioRender);
        self.audioRender = NULL;
    }
    
    if (self.videoCaptureThreadId) {
        pthread_join(self.videoCaptureThreadId, NULL);
        self.videoCaptureThreadId = 0;
    }
    
    if (self.hChannel) {
        MWCloseChannel(self.hChannel);
        self.hChannel = NULL;
    }
    
    if (self.selectedDeviceItem) {
        [self.selectedDeviceItem setState:NSControlStateValueOff];
        self.selectedDeviceItem = nil;
    }
}

- (void)initParameters {
    self.width         = 1920;
    self.height        = 1080;
    self.frameDuration = 166667;
    self.fourcc        = MWFOURCC_NV12;

    self.samplerate    = 48000;
    self.audioChannels = 2;
    self.bitsPerSample = 16;

    self.audioRender   = NULL;

    self.running              = false;
    self.hChannel             = NULL;
    self.videoCaptureThreadId = 0;
    self.audioCaptureThreadId = 0;

    self.captureFps      = 0.0;
    self.lastTimestamp   = 0;
    self.videoFrameCount = 0;
    
    self.pixelFormat = kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange;
}

- (void)initDeviceMenu {
    int count = MWGetChannelCount();
    for (int i = 0; i < count; i++) {
        NSString *itemTitle = nil;
        
        MWCAP_CHANNEL_INFO mci = { 0 };
        MWGetChannelInfoByIndex(i, &mci);
        if (strcmp(mci.szFamilyName, "Pro Capture") == 0) {
            itemTitle = [NSString stringWithFormat:@"%02d-%d %s", mci.byBoardIndex, mci.byChannelIndex, mci.szProductName];
            NSMenuItem *deviceItem = [self.deviceMenu addItemWithTitle:itemTitle action:@selector(startCapture:) keyEquivalent:@""];
            [deviceItem setTag:i];
        }
    }
}

- (void)createPreviewLayer {
    if (!self.videoLayer) {
        self.videoLayer = [[AVSampleBufferDisplayLayer alloc] init];
        self.videoLayer.frame = CGRectMake(0, self.statusLabel.frame.size.height,
                                           self.view.frame.size.width, self.view.frame.size.height - self.statusLabel.frame.size.height);
        self.videoLayer.bounds = CGRectMake(0, self.statusLabel.frame.size.height,
                                            self.view.bounds.size.width, self.view.bounds.size.height - self.statusLabel.frame.size.height);
        self.videoLayer.autoresizingMask = NSViewWidthSizable | NSViewHeightSizable;
        self.videoLayer.videoGravity = AVLayerVideoGravityResizeAspect;
        self.videoLayer.backgroundColor = [NSColor blackColor].CGColor;
        [[self.view layer] addSublayer:self.videoLayer];
        
        self.view.window.delegate = self;
    }
}

- (void)parseArgs {
    NSArray *args = [[NSProcessInfo processInfo] arguments];
    for (int i=0; i<args.count; i++) {
        NSString *item = [args objectAtIndex:i];
        const char *ptr = strstr([item UTF8String], "-w=");
        if (ptr) {
            self.width = atoi(ptr+strlen("-w="));
            continue;
        }
        
        ptr = strstr([item UTF8String], "-h=");
        if (ptr) {
            self.height = atoi(ptr+strlen("-h="));
            continue;
        }
        
        ptr = strstr([item UTF8String], "-f=");
        if (ptr) {
            self.frameDuration = atoi(ptr+strlen("-f="));
            continue;
        }
        
        ptr = strstr([item UTF8String], "-v=");
        if (ptr) {
           self.viewEnable = atoi(ptr+strlen("-v="));
           continue;
        }

    }
}

- (void)viewDidLoad {
    [self.view setWantsLayer:YES];
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    //base UI
    {
        self.viewEnable = 1;
        [self performSelector:@selector(createPreviewLayer) withObject:nil afterDelay:0];
        
        NSMenu *mainMenu = [NSApp mainMenu];
        
        NSMenuItem *mainMenuDeviceItem = [[NSMenuItem alloc] init];
        [mainMenu addItem:mainMenuDeviceItem];
        self.deviceMenu = [[NSMenu alloc] initWithTitle:@"Device"];
        [mainMenuDeviceItem setSubmenu:self.deviceMenu];
    }
    
    MWCaptureInitInstance();
    MWRefreshDevice();
    
    [self initParameters]; [self parseArgs];
    [self initDeviceMenu];
    
    if (self.deviceMenu.numberOfItems > 0) {
        [self performSelector:@selector(startCapture:) withObject:[self.deviceMenu itemAtIndex:0] afterDelay:0];
    }
    
    self.statusTimer = [NSTimer scheduledTimerWithTimeInterval:0.5 target: self selector:@selector(statusTimerProc:) userInfo:nil repeats:YES];
}

- (BOOL)windowShouldClose:(NSWindow *)sender {
    [self stopCapture];
    
    if (self.statusTimer) {
        [self.statusTimer invalidate];
    }
    
    MWCaptureExitInstance();
    
    return YES;
}

- (void)viewDidDisappear {
}

- (void)setRepresentedObject:(id)representedObject {
    [super setRepresentedObject:representedObject];

    // Update the view, if already loaded.
}

- (void) statusTimerProc: (NSTimer *) timer {
    char strFourcc[5] = { 0 };
    *(uint32_t*)strFourcc = self.fourcc;
    [self.statusLabel setStringValue:[NSString stringWithFormat:@" %d x %d %s %.2f FPS ", self.width, self.height, strFourcc, self.captureFps]];
}

@end
